<?php
/*
* Project:     Morgdkp2 eqdkp plus plug-in
* License:     Creative Commons - Attribution-Noncommercial-Share Alike 3.0 Unported
* Link:    http://creativecommons.org/licenses/by-nc-sa/3.0/
* -----------------------------------------------------------------------
* Began:       2013
* -----------------------------------------------------------------------
* @author      BlueWarlord
* @copyright   2013-
* Based upon the EQdkp-Plus epgpimport plug-in and Morgalm's original morgdkp 2 plug-in for eqkdp plus 0.6
*/


if(!defined('EQDKP_INC')) {
	header('HTTP/1.0 Not Found');
	exit;
}
$eqdkp_root_path = './../../../';
include_once($eqdkp_root_path.'common.php');

if(!class_exists('morgdkp2_parser')) {
	class morgdkp2_parser extends gen_class {
		public static $shortcuts = array('in', 'pdh', 'pdc', 'user', 'config', 'time');
		public $webversion, $Switches, $RAID, $DKP, $dkpchange, $passalts;
		private $pdl;
		
		public function __construct() {
			$this->passalts = array();
			$this->dkpchange = array();
			$this->webversion = "3.3";
			$this->DKP = array();
			$this->RAID = array();
			$this->Switches = array();
			include_once('../includes/morgdkp2_functions.class.php');
			$this->func = register('morgdkp2_functions');	
			// Sort out the debug logger
			$this->pdl = registry::register('plus_debug_logger');
			$this->pdl->register_type("morgdkp2", null, null, null, "morgdkp2.log");
			$this->pdl->log("morgdkp2", "Constructing class" );
		}
		
		public function parse($strLog){
			$this->pdl->log("morgdkp2", "Starting parsing ...");  
			$temp = $this->xmlstringtoarray($strLog);
			//$this->pdl->log("morgdkp2", 'DKP array:',$this->pdl->format_array($temp));
			$temp = $this->stripslashes_deep($temp);      
			$this->DKP = $temp['MorgDKP2'];
			$this->pdl->log("morgdkp2", 'DKP string:'.$this->pdl->format_array($this->DKP));
			
			if ($this->DKP['version'] != $this->webversion) {
				// Error return false
				return false;
			} else {
				foreach ($this->DKP as $eventtype => $table) {
					switch ($eventtype) {
						case 'Aliases':
							$this->addAliases($table);
							break;
						case 'DKPChanges':
							$this->changeDKP($table);
							break;
						case 'Raids':                
							$this->addRaids($table);
							//$this->pdl->log("morgdkp2", 'Raid:',$this->pdl->format_array($this->RAID)); 
							//$this->pdl->log("morgdkp2", 'Raid session:',$this->pdl->format_array($_SESSION['RAID'])); 
							break;
					}
				}
			}
			return true;
		}

		private function xmlstringtoarray($xml) {      
			$return = array(); 
			$stack = array();
			$list = array();
			preg_match_all("/<(.*?)>/", $xml, $opentag);
			preg_match_all("/>(.*?)</", $xml, $data);
			
			foreach($opentag[1] as $tagnum => $tagname) {
				if (preg_match("/\//", $tagname)) {
					if (preg_match("/\/key/", $tagname)) $tagname = preg_replace("/key/", "", $tagname);
						$list[] = array('tag'=>$tagname, 'data'=>'', 'status'=>"closed");
					} else {
						if (preg_match("/key/", $tagname)) $tagname = preg_replace("/key/", "", $tagname);
						if ($data[1][$tagnum] != '') {
							$list[] = array('tag'=>$tagname, 'data'=>$data[1][$tagnum], 'status'=>"complete");
						} else {
							$list[] = array('tag'=>$tagname, 'data'=>'', 'status'=>"open");
						}
					}
			}
			
			$lastclosed = "";
			foreach($list as $val) {
				if($val['status'] == "open") {
					array_push($stack, $val['tag']);
				} elseif($val['status'] == "closed") {
					if ($lastclosed != $val['tag']) {
						array_pop($stack);
						$lastclosed = "/" . $val['tag'];
					}
				} elseif($val['status'] == "complete") {
					array_push($stack, $val['tag']);
					$this->setArrayValue($return, $stack, $val['data']);
					array_pop($stack);
					$lastclosed = "/" . $val['tag'];
				}
			}
			return $return;
		}

		private function setArrayValue(&$array, $stack, $value) {
			if ($stack) {
				$key = array_shift($stack);
				$this->setArrayValue($array[$key], $stack, $value);
				return $array;
			} else $array = $value;
			
		}
		
		private function addRaids($raids) {
			$i = 0;
			$this->pdl->log("morgdkp2", 'Adding raids:',$this->pdl->format_array($raids));
			if (!empty($raids)) {
				$RAID = array();
				foreach ($raids as $raid) {
					$RAID[$i]['zone'] = $raid['zone'];
					$RAID[$i]['date'] = $raid['date'];
					$RAID[$i]['eqdkp'] = $raid['eqdkp'];
					$RAID[$i]['pool'] = $raid['pool'];
					$RAID[$i]['prefix'] = $raid['prefix'];
					foreach ($raid['Attendees'] as $attend) {
						$name = $attend['name'];
						$RAID[$i]['attendees'][$name] = array('race'=>$attend['race'], 'class'=>$attend['class'], 'level'=>$attend['level']);
					}
					$eventindex = 0;
					if (isset($raid['Bosskills'])) {
						foreach ($raid['Bosskills'] as $boss) {
							$bossname = $boss['name'];
							$bossnote = $boss['note'];
							$attend = $boss['attendees'];
							$attend = $this->func->CheckAlias($attend, $raid['eqdkp'], $raid['prefix']);
							if (!empty($attend)) {
								asort($attend);
								$attend = array_unique($attend);
								$RAID[$i]['events'][$eventindex] = array('name'=>$bossname, 'note'=>$bossnote,'value'=>$boss['value'], 'time'=>$boss['time'], 'attendees'=>$attend, 'trash'=>$boss['trash'], 'index'=>$eventindex);
								$eventindex++;
							}
						}
					}
					$trashdone = false;
					if (isset($RAID[$i]['events'])) {
						foreach ($RAID[$i]['events'] as $event) {
							$trashcounter = 0;
							$lootnumber = 0;
							if (isset($raid['Loot'])) {
								foreach ($raid['Loot'] as $loot) {
									if ($event['note'] == $loot['Boss']) {
										$index = $event['index'];
										$isalias = $this->func->CheckAlias(array('0' => $loot['Player']));
										$loot['Player'] = $isalias[0];
										$RAID[$i]['loot'][$index][$lootnumber] = $loot;
										$lootnumber++;
									}
									if (!$trashdone and $loot['Boss'] == "Trash mob") {
										$isalias = $this->func->CheckAlias(array('0' => $loot['Player']));
										$loot['Player'] = $isalias[0];
										$RAID[$i]['loot']['Trash mob'][$trashcounter] = $loot;
										$trashcounter++;
									}
								}
								$trashdone = true;
							}
						}     
						$i++;
					}
				}
				if (!empty($RAID)) {
					$this->RAID = $RAID;
					$this->pdc->put('MDKP_RAID', $RAID);
					$this->Switches['raids'] = true;
				}    
			} 
		}


		private function stripslashes_deep($value) {
			$value = is_array($value) ? array_map(array('morgdkp2_parser','stripslashes_deep'), $value) :  stripslashes($value);
			return $value;
		}
	    
		private function addAliases($aliases) {
			if (!empty($aliases)) {
				$newalts = array();
				foreach ($aliases as $table) {
					$main = $table['main'];
					$alt = $table['alt'];
					$memberid = $this->func->FetchMember($main);
					if ($memberid) {
						$newalts[] = array('alt'=>$alt, 'main'=>$main, 'eqdkp'=>$table['eqdkp'], 'pool'=>$table['pool'], 'prefix'=>$table['prefix'], 'id'=>$memberid, 'action'=>$table['action']);
						$this->Switches['alias'] = true;
					} else {
						$newalts[] = array('alt'=>$alt, 'main'=>$main, 'eqdkp'=>$table['eqdkp'], 'pool'=>$table['pool'], 'prefix'=>$table['prefix'], 'id'=>$memberid, 'action'=>'NO MAIN!');
						$this->Switches['alias'] = true;
					}
				}
			}
			if (!empty($newalts)) {
				$this->pdc->put('MDKP_PASSALTS', $newalts);
				$this->passalts = $newalts;
			}
		} // End of addAlias()
	
		private function changeDKP($changes) {
			if (!empty($changes)) {
				$needchanged = array();
				foreach ($changes as $change) {
					$name = $change['name'];
					$members = array();            
					foreach ($change['members'] as $memberkey => $member) {
						$members[$memberkey] = $member;
					}
					$members = $this->func->CheckAlias($members);
					$needchanged[] = array('addtype'=>$change['addtype'], 'name'=>$name, 'value'=>$change['value'], 'eqdkp'=>$change['eqdkp'], 'pool'=>$change['pool'], 'prefix'=>$change['prefix'], 'event'=>$change['event'], 'members'=>$members);
				}
			}
			if (!empty($needchanged)) {
				$this->pdc->put('MDKP_DKPCHANGE', $needchanged);
				$this->dkpchange = $needchanged;
				$this->Switches['dkpchange'] = true;
			}
		} // End of changeDKP()
		
					
 	 } // End of morgdkp2_parser class
} // End of enclosing if

if(version_compare(PHP_VERSION, '5.3.0', '<')) {
	registry::add_const('short_epgp_parser', epgp_parser::$shortcuts);
}
?>